@@ -6,14 +6,12 @@ require Carp;
 # So one can use Carp::carp "$message" - without the parenthesis.
 sub Carp::carp;
 use IO::All::Base -base;
-our $VERSION = '0.36';
+our $VERSION = '0.39';
 use File::Spec();
 use Symbol();
 use Fcntl;
 our @EXPORT = qw(io);
 
-sub io { IO::All->new(@_) }
-
 #===============================================================================
 # Object creation and setup methods
 #===============================================================================
@@ -54,7 +52,8 @@ sub mldbm {my $self = shift; require IO::All::MLDBM; IO::All::MLDBM::mldbm($self
 
 sub autoload {my $self = shift; $autoload }
 
-sub AUTOLOAD {my $self = shift;
+sub AUTOLOAD {
+    my $self = shift;
     my $method = $IO::All::AUTOLOAD;
     $method =~ s/.*:://;
     my $pkg = ref($self) || $self;
@@ -359,12 +358,15 @@ sub overload_scalar_to_any {
 field 'package';
 field _binary => undef;
 field _binmode => undef;
+field _strict => undef;
+field _encoding => undef;
 field _utf8 => undef;
 field _handle => undef;
 
 #===============================================================================
 # Public Accessors
 #===============================================================================
+field constructor => undef;
 chain block_size => 1024;
 chain errors => undef;
 field io_handle => undef;
@@ -387,6 +389,7 @@ option 'confess';
 option 'lock';
 option 'rdonly';
 option 'rdwr';
+option 'strict';
 
 #===============================================================================
 # IO::Handle proxy methods
@@ -412,22 +415,69 @@ proxy_open 'getc';
 # File::Spec Interface
 #===============================================================================
 sub canonpath {my $self = shift; File::Spec->canonpath($self->pathname) } 
-sub catdir {my $self = shift; $self->new->dir(File::Spec->catdir(@_)) } 
-sub catfile {my $self = shift; $self->new->file(File::Spec->catfile(@_)) } 
+sub catdir {
+    my $self = shift;
+    my @args = grep defined, $self->name, @_;
+    $self->constructor->()->dir(File::Spec->catdir(@args));
+} 
+sub catfile {
+    my $self = shift;
+    my @args = grep defined, $self->name, @_;
+    $self->constructor->()->file(File::Spec->catfile(@args));
+} 
 sub join {my $self = shift; $self->catfile(@_) } 
-sub curdir {my $self = shift; $self->new->dir(File::Spec->curdir) } 
-sub devnull {my $self = shift; $self->new->file(File::Spec->devnull) } 
-sub rootdir {my $self = shift; $self->new->dir(File::Spec->rootdir) } 
-sub tmpdir {my $self = shift; $self->new->dir(File::Spec->tmpdir) } 
-sub updir {my $self = shift; $self->new->dir(File::Spec->updir) } 
-sub case_tolerant {my $self = shift; File::Spec->case_tolerant } 
-sub is_absolute {my $self = shift; File::Spec->file_name_is_absolute($self->pathname) }
-sub path {my $self = shift; map { $self->new->dir($_) } File::Spec->path } 
-sub splitpath {my $self = shift; File::Spec->splitpath($self->pathname) } 
-sub splitdir {my $self = shift; File::Spec->splitdir($self->pathname) } 
-sub catpath {my $self = shift; $self->new(File::Spec->catpath(@_)) } 
-sub abs2rel {my $self = shift; File::Spec->abs2rel($self->pathname, @_) } 
-sub rel2abs {my $self = shift; File::Spec->rel2abs($self->pathname, @_) }
+sub curdir {
+    my $self = shift;
+    $self->constructor->()->dir(File::Spec->curdir);
+} 
+sub devnull {
+    my $self = shift;
+    $self->constructor->()->file(File::Spec->devnull);
+} 
+sub rootdir {
+    my $self = shift;
+    $self->constructor->()->dir(File::Spec->rootdir);
+} 
+sub tmpdir {
+    my $self = shift;
+    $self->constructor->()->dir(File::Spec->tmpdir);
+} 
+sub updir {
+    my $self = shift;
+    $self->constructor->()->dir(File::Spec->updir);
+} 
+sub case_tolerant {
+    my $self = shift;
+    File::Spec->case_tolerant;
+} 
+sub is_absolute {
+    my $self = shift;
+    File::Spec->file_name_is_absolute($self->pathname);
+}
+sub path {
+    my $self = shift;
+    map { $self->constructor->()->dir($_) } File::Spec->path;
+} 
+sub splitpath {
+    my $self = shift;
+    File::Spec->splitpath($self->pathname);
+} 
+sub splitdir {
+    my $self = shift;
+    File::Spec->splitdir($self->pathname);
+} 
+sub catpath {
+    my $self = shift;
+    $self->constructor->(File::Spec->catpath(@_));
+} 
+sub abs2rel {
+    my $self = shift;
+    File::Spec->abs2rel($self->pathname, @_);
+} 
+sub rel2abs {
+    my $self = shift;
+    File::Spec->rel2abs($self->pathname, @_);
+}
 
 #===============================================================================
 # Public IO Action Methods
@@ -638,10 +688,26 @@ sub slurp {
 
 sub utf8 {
     my $self = shift;
-    return $self if $] < 5.008;
+    if ($] < 5.008) {
+        die "IO::All -utf8 not supported on Perl older than 5.8";
+    }
     CORE::binmode($self->io_handle, ':utf8')
       if $self->is_open;
     $self->_utf8(1);
+    $self->encoding('utf8');
+    return $self;
+}
+
+sub encoding {
+    my $self = shift;
+    my $encoding = shift
+      or die "No encoding value passed to IO::All::encoding";
+    if ($] < 5.008) {
+        die "IO::All -encoding not supported on Perl older than 5.8";
+    }
+    CORE::binmode($self->io_handle, ":$encoding")
+      if $self->is_open;
+    $self->_encoding($encoding);
     return $self;
 }
 
@@ -712,12 +778,15 @@ sub copy {
 
 sub set_binmode {
     my $self = shift;
-    CORE::binmode($self->io_handle) 
-      if $self->_binary;
-    CORE::binmode($self->io_handle, ":utf8")
-      if $self->_utf8;
-    CORE::binmode($self->io_handle, $self->_binmode)
-      if $self->_binmode;
+    if (my $encoding = $self->_encoding) {
+        CORE::binmode($self->io_handle, ":encoding($encoding)");
+    }
+    elsif ($self->_binary) {
+        CORE::binmode($self->io_handle);
+    }
+    elsif ($self->_binmode) {
+        CORE::binmode($self->io_handle, $self->_binmode);
+    }
     return $self;
 }
 
